﻿using System;
using System.Collections.Generic;
using System.Net;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Threading.Tasks;

using TestLibrary;

namespace TestClient
{
    /// <summary>
    /// 프로그램
    /// </summary>
    class Program
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 프로그램 시작하기 - Main()

        /// <summary>
        /// 프로그램 시작하기
        /// </summary>
        static void Main()
        {
            User user = new User { UserName = "admin", Password = "1234"};

            string token = Login("https://localhost:44316/Login", user).GetAwaiter().GetResult();

            Console.WriteLine($"JWT 토큰 : {token}");

            Console.WriteLine("--------------------------------------------------");

            IEnumerable<WeatherForecast> result = Test("https://localhost:44316/WeatherForecast", token).GetAwaiter().GetResult();

            if(result != null)
            {
                foreach(WeatherForecast item in result)
                {
                    Console.WriteLine(item.Summary);
                }
            }
            
            Console.WriteLine("--------------------------------------------------");

            Console.WriteLine("아무 키나 눌러 주시기 바랍니다.");

            Console.ReadKey(true);
        }

        #endregion

        #region HTTP 클라이언트 구하기 - GetHTTPClient(useCookie, baseAddress, tokenShema, token)

        /// <summary>
        /// HTTP 클라이언트 구하기
        /// </summary>
        /// <param name="useCookie">쿠키 사용 여부</param>
        /// <param name="baseAddress">기본 주소</param>
        /// <param name="tokenShema">토큰 스키마</param>
        /// <param name="token">토큰</param>
        /// <returns>HTTP 클라이언트</returns>
        private static HttpClient GetHTTPClient(bool useCookie, string baseAddress, string tokenShema, string token)
        {
            HttpClient client;

            if(useCookie)
            {
                HttpClientHandler httpClientHandler = new HttpClientHandler();

                httpClientHandler.UseCookies      = true;
                httpClientHandler.CookieContainer = new CookieContainer();
    
                client = new HttpClient(httpClientHandler);
            }
            else
            {
                client = new HttpClient();
            }

            client.DefaultRequestHeaders.Accept.Clear();
            client.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

            if(!string.IsNullOrWhiteSpace(baseAddress))
            {
                client.BaseAddress = new Uri(baseAddress);
            }

            if(!string.IsNullOrWhiteSpace(tokenShema))
            {
                client.DefaultRequestHeaders.Authorization = new AuthenticationHeaderValue(tokenShema, token);
            }

            return client;
        }

        #endregion
        #region 로그인하기 - Login(url, user)

        /// <summary>
        /// 로그인하기
        /// </summary>
        /// <param name="url">URL</param>
        /// <param name="user">사용자</param>
        /// <returns>JWT 토큰</returns>
        private static async Task<string> Login(string url, User user)
        {
            using(HttpClient client = GetHTTPClient(false, url, null, null))
            {
                var response = await client.PostAsJsonAsync<User>(url, user);

                if(response.IsSuccessStatusCode)
                {
                    string token = await response.Content.ReadAsAsync<string>();

                    return token;
                }

                return null;

            }
        }

        #endregion
        #region 테스트하기 - Test(url, token, test)

        /// <summary>
        /// 테스트하기
        /// </summary>
        /// <param name="url">URL</param>
        /// <param name="token">JWT 토큰</param>
        /// <param name="test">테스트</param>
        /// <returns>테스트 결과</returns>
        private static async Task<IEnumerable<WeatherForecast>> Test(string url, string token)
        {
            using(HttpClient client = GetHTTPClient(false, url, "Bearer", token))
            {
                var response = await client.GetAsync(url);

                if(response.IsSuccessStatusCode)
                {
                    IEnumerable<WeatherForecast> result = await response.Content.ReadAsAsync<IEnumerable<WeatherForecast>>();

                    return result;
                }
                else // http://로 호출한 경우 https://로 다시 호출한다.
                {
                    string finalRequestURL = response.RequestMessage.RequestUri.AbsoluteUri;

                    if(finalRequestURL != url)
                    {
                        response = await client.GetAsync(finalRequestURL);

                        if(response.IsSuccessStatusCode)
                        {
                            IEnumerable<WeatherForecast> result = await response.Content.ReadAsAsync<IEnumerable<WeatherForecast>>();

                            return result;
                        }
                        else
                        {
                            return null;
                        }
                    }
                    else
                    {
                        return null;
                    }
                }
            }
        }

        #endregion
    }
}