﻿using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Configuration;
using Microsoft.IdentityModel.Tokens;
using System;
using System.Collections.Generic;
using System.IdentityModel.Tokens.Jwt;
using System.Security.Claims;
using System.Text;

using TestLibrary;
using TestServer.Tools;

namespace TestServer.Controllers
{
    /// <summary>
    /// 로그인 컨트롤러
    /// </summary>
    [ApiController]
    [Route("[controller]")]
    public class LoginController : ControllerBase
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 사용자 딕셔너리
        /// </summary>
        private static Dictionary<string, User> _userDictionary = new Dictionary<string, User>();

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 구성
        /// </summary>
        private readonly IConfiguration configuration;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Static

        #region 생성자 - LoginController()

        /// <summary>
        /// 생성자
        /// </summary>
        static LoginController()
        {
            _userDictionary.Add("admin", new User { UserName = "admin", Password = "1234", UserRole = Policy.Administrator });
            _userDictionary.Add("user1", new User { UserName = "user1", Password = "1234", UserRole = Policy.User          });
            _userDictionary.Add("user2", new User { UserName = "user2", Password = "1234", UserRole = Policy.User          });
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - LoginController(configuration)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="configuration">구성</param>
        public LoginController(IConfiguration configuration)
        {
            this.configuration = configuration;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 로그인하기 - Login(loginUser)

        /// <summary>
        /// 로그인하기
        /// </summary>
        /// <param name="loginUser">로그인 사용자</param>
        /// <returns>액션 결과</returns>
        [HttpPost]
        [AllowAnonymous]
        public IActionResult Login([FromBody] User loginUser)
        {
            IActionResult actionResult = Unauthorized();

            User user = AuthenticateUser(loginUser);

            if(user != null)
            {
                string token = GenerateJWTToken(user);

                actionResult = Ok(token);
            }

            return actionResult;
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 사용자 인증하기 - AuthenticateUser(loginUser)

        /// <summary>
        /// 사용자 인증하기
        /// </summary>
        /// <param name="loginUser">로그인 사용자</param>
        /// <returns>인증 사용자</returns>
        private User AuthenticateUser(User loginUser)
        {
            if(_userDictionary.ContainsKey(loginUser.UserName))
            {
                User user = _userDictionary[loginUser.UserName];

                if(user.Password == loginUser.Password)
                {
                    return user;
                }
                else
                {
                    return null;
                }
            }
            else
            {
                return null;
            }
        }

        #endregion
        #region JWT 토큰 생성하기 - GenerateJWTToken(user)

        /// <summary>
        /// JWT 토큰 생성하기
        /// </summary>
        /// <param name="user">사용자</param>
        /// <returns>JWT 토큰</returns>
        private string GenerateJWTToken(User user)
        {
            byte[] keyByteArray = Encoding.UTF8.GetBytes(configuration["JWT:SecretKey"]);

            SymmetricSecurityKey symmetricSecurityKey = new SymmetricSecurityKey(keyByteArray);

            SigningCredentials credentials = new SigningCredentials(symmetricSecurityKey, SecurityAlgorithms.HmacSha256);

            Claim[] claimArray = new[]
            {
                new Claim(JwtRegisteredClaimNames.Sub, user.UserName            ),
                new Claim("role"                     , user.UserRole            ),
                new Claim(JwtRegisteredClaimNames.Jti, Guid.NewGuid().ToString())
            };

            JwtSecurityToken token = new JwtSecurityToken
            (
                issuer            : configuration["JWT:Issuer"],
                audience          : configuration["JWT:Audience"],
                claims            : claimArray,
                expires           : DateTime.Now.AddHours(1),
                signingCredentials: credentials
            );

            return new JwtSecurityTokenHandler().WriteToken(token);
        }

        #endregion
    }
}