using Microsoft.AspNetCore.Authentication.JwtBearer;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Microsoft.IdentityModel.Tokens;
using Microsoft.OpenApi.Models;
using System;
using System.Text;

using TestServer.Tools;

namespace TestServer
{
    /// <summary>
    /// 
    /// </summary>
    public class Startup
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region  - Configuration

        /// <summary>
        /// 
        /// </summary>
        public IConfiguration Configuration { get; }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region  - Startup(configuration)

        /// <summary>
        /// 
        /// </summary>
        /// <param name="configuration"></param>
        public Startup(IConfiguration configuration)
        {
            Configuration = configuration;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region  ÷ ϱ - ConfigureServices(serviceCollection)

        /// <summary>
        ///  ÷ ϱ
        /// </summary>
        /// <param name="serviceCollection"> ÷</param>
        public void ConfigureServices(IServiceCollection serviceCollection)
        {
            serviceCollection.AddAuthentication(JwtBearerDefaults.AuthenticationScheme)
                .AddJwtBearer
                (
                    option =>
                    {
                        option.RequireHttpsMetadata      = false;
                        option.SaveToken                 = true;
                        option.TokenValidationParameters = new TokenValidationParameters
                        {
                            ValidateIssuer           = true,
                            ValidateAudience         = true,
                            ValidateLifetime         = true,
                            ValidateIssuerSigningKey = true,
                            ValidIssuer              = Configuration["JWT:Issuer"],
                            ValidAudience            = Configuration["JWT:Audience"],
                            IssuerSigningKey         = new SymmetricSecurityKey(Encoding.UTF8.GetBytes(Configuration["JWT:SecretKey"])),
                            ClockSkew                = TimeSpan.Zero
                        };
                    }
                );

            serviceCollection.AddAuthorization
            (
                option =>
                {
	                option.AddPolicy(Policy.Administrator, Policy.GetAdministratorAuthorizationPolicy());
	                option.AddPolicy(Policy.User         , Policy.GetUserAuthorizationPolicy()         );
                }
            );

            serviceCollection.AddControllers();

            serviceCollection.AddSwaggerGen
            (
                option =>
                {
                    option.SwaggerDoc("v1", new OpenApiInfo { Title = "TestServer", Version = "v1" });
                }
            );
        }

        #endregion
        #region ϱ - Configure(applicationBuilder, environment)

        /// <summary>
        /// ϱ
        /// </summary>
        /// <param name="applicationBuilder">ø̼ </param>
        /// <param name="environment">ȯ</param>
        public void Configure(IApplicationBuilder applicationBuilder, IWebHostEnvironment environment)
        {
            if(environment.IsDevelopment())
            {
                applicationBuilder.UseDeveloperExceptionPage();

                applicationBuilder.UseSwagger();

                applicationBuilder.UseSwaggerUI(c => c.SwaggerEndpoint("/swagger/v1/swagger.json", "TestServer v1"));
            }

            applicationBuilder.UseHttpsRedirection();

            applicationBuilder.UseRouting();

            applicationBuilder.UseAuthentication();

            applicationBuilder.UseAuthorization();

            applicationBuilder.UseEndpoints
            (
                endpoints =>
                {
                    endpoints.MapControllers();
                }
            );
        }

        #endregion
    }
}